package org.dbxml.core.corba;

/*
 * dbXML License, Version 1.0
 *
 *
 * Copyright (c) 1999-2001 The dbXML Group, L.L.C.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        The dbXML Group (http://www.dbxml.com/)."
 *    Alternately, this acknowledgment may appear in the software
 *    itself, if and wherever such third-party acknowledgments normally
 *    appear.
 *
 * 4. The names "dbXML" and "The dbXML Group" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    info@dbxml.com.
 *
 * 5. Products derived from this software may not be called "dbXML",
 *    nor may "dbXML" appear in their name, without prior written
 *    permission of The dbXML Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE DBXML GROUP OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: CollectionManagerServant.java,v 1.18 2001/06/20 00:20:27 tom Exp $
 */

import org.dbxml.core.corba.db.*;
import org.dbxml.core.FaultCodes;

/**
 * CollectionManagerServant CORBA wrapper for org.dbxml.core.Collection
 * Provides administrative access to Collection
 * For detailed documentation
 * @see org.dbxml.core.Collection
 */

public class CollectionManagerServant extends CollectionManagerPOA {
   org.dbxml.core.Collection collection;

   public CollectionManagerServant(org.dbxml.core.Collection collection) {
      this.collection = collection;
   }

   public String getName () throws APIException {
      try {
         return collection.getName();
      }
      catch (Exception e) {
         throw FaultCodes.createAPIException(e);
      }
   }

   public String getCanonicalName () throws APIException {
      try {
         return collection.getCanonicalName();
      }
      catch (Exception e) {
         throw FaultCodes.createAPIException(e);
      }
   }

   public org.dbxml.core.corba.db.Collection createCollection (
      String path, org.dbxml.core.corba.db.EncodedBuffer config)
         throws APIException {
      try {
         /*Collection col = collection.createCollection(nam,
            EncodedBufferConverter.convertToConfiguration(config));
         if (col == null) {
            // TODO: throw an exception
         }*/

         return new CollectionServant(collection.createCollection(path,
            EncodedBufferConverter.convertToConfiguration(config)))._this(_orb());
      }
      catch (Exception e) {
         throw FaultCodes.createAPIException(e);
      }
   }

   public void dropCollection (String nam) throws APIException {
      try {
         collection.dropCollection(collection.getCollection(nam));
      }
      catch (Exception e) {
         throw FaultCodes.createAPIException(e);
      }
   }

   public String[] listIndexers () throws APIException {
      try {
         if (collection == null)
            System.out.println("NULL");
            
         return collection.listIndexers();
      }
      catch (Exception e) {
         throw FaultCodes.createAPIException(e);
      }
   }

   public void createIndexer (EncodedBuffer config) throws APIException {
      try {
         collection.createIndexer(
            EncodedBufferConverter.convertToConfiguration(config));
      }
      catch (Exception e) {
         throw FaultCodes.createAPIException(e);
      }
   }

   public void dropIndexer (String nam) throws APIException {
      try {
         collection.dropIndexer(collection.getIndexer(nam));
      }
      catch (Exception e) {
         throw FaultCodes.createAPIException(e);
      }
   }

   public String[] listXMLObjects () throws APIException {
      try {
         return collection.listXMLObjects();
      }
      catch (Exception e) {
         throw FaultCodes.createAPIException(e);
      }
   }

   public void createXMLObject(EncodedBuffer config) throws APIException {
      try {
         collection.createXMLObject(
            EncodedBufferConverter.convertToConfiguration(config));
      }
      catch (Exception e) {
         throw FaultCodes.createAPIException(e);
      }
   }

   public void dropXMLObject(String nam) throws APIException {
      try {
         collection.dropXMLObject(collection.getXMLObject(nam));
      }
      catch (Exception e) {
         throw FaultCodes.createAPIException(e);
      }
   }

   /*
    * Releases all resources associated with this object. This method must be
    * called when usage of the object is complete. If it is not called the
    * server side servant will not be properly released.
    */
   public void remove() {
      try {
         _poa().deactivate_object(_object_id());
      }
      catch (Exception e) {
         org.dbxml.Debug.printStackTrace(e);
      }
   }
}
