package org.dbxml.tests;

/*
 * dbXML License, Version 1.0
 *
 *
 * Copyright (c) 1999-2001 The dbXML Group, L.L.C.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        The dbXML Group (http://www.dbxml.com/)."
 *    Alternately, this acknowledgment may appear in the software
 *    itself, if and wherever such third-party acknowledgments normally
 *    appear.
 *
 * 4. The names "dbXML" and "The dbXML Group" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    info@dbxml.com.
 *
 * 5. Products derived from this software may not be called "dbXML",
 *    nor may "dbXML" appear in their name, without prior written
 *    permission of The dbXML Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE DBXML GROUP OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: BTreeTest.java,v 1.6 2001/05/23 00:35:47 tom Exp $
 */

import org.dbxml.core.data.*;
import org.dbxml.core.filer.*;
import java.io.*;

/**
 * BTreeTest excercises BTree functionality.
 */

public class BTreeTest {
   public static void main(String[] args) {
      try {
         if ( args[0].equals("create") ) {
            BTree f = new BTree(new File(args[1]));
            f.create();
         }
         else if ( args[0].equals("import") ) {
            BTree f = new BTree(new File(args[1]));
            f.open();
            FileInputStream fis = new FileInputStream(args[2]);
            DataInputStream dis = new DataInputStream(fis);
            int i = 0;
            while ( dis.available() > 0 ) {
               String val = dis.readLine().trim();
               if ( val.length() > 0 ) {
                  if ( val.indexOf("/") >= 0 )
                     val = val.substring(0, val.indexOf("/"));
                  f.addValue(new Value(val), i++);
               }
            }
            fis.close();
            f.close();
         }
         else if ( args[0].equals("export") ) {
            BTree f = new BTree(new File(args[1]));
            f.open();
            FileOutputStream fos = new FileOutputStream(args[2]);
            final PrintStream ps = new PrintStream(fos);
            f.query(null, new BTreeCallback() {
               public boolean indexInfo(Value value, long pointer) {
                  ps.println(value);
                  return true;
               }
            });
            fos.close();
            f.close();
         }
         else if ( args[0].equals("find") ) {
            BTree f = new BTree(new File(args[1]));
            f.open();
            long ptr = f.findValue(new Value(args[2]));
            System.out.println("Result = "+ptr);
            f.close();
         }
      }
      catch ( Exception e ) {
         System.out.println("usage:");
         System.out.println("   create <btree>");
         System.out.println("      Create a new B+Tree file");
         System.out.println();
         System.out.println("   import <btree> <import file>");
         System.out.println("      Import text line by line into B+Tree");
         System.out.println();
         System.out.println("   export <btree> <export file>");
         System.out.println("      Export values line by line from B+Tree");
         System.out.println();
         System.out.println("   find <btree> <value>");
         System.out.println("      Find value in the B+Tree");
      }
   }
}
