package org.dbxml.xml.dom;

/*
 * dbXML License, Version 1.0
 *
 *
 * Copyright (c) 1999-2001 The dbXML Group, L.L.C.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        The dbXML Group (http://www.dbxml.com/)."
 *    Alternately, this acknowledgment may appear in the software
 *    itself, if and wherever such third-party acknowledgments normally
 *    appear.
 *
 * 4. The names "dbXML" and "The dbXML Group" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    info@dbxml.com.
 *
 * 5. Products derived from this software may not be called "dbXML",
 *    nor may "dbXML" appear in their name, without prior written
 *    permission of The dbXML Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE DBXML GROUP OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: CompressedNode.java,v 1.8 2001/05/23 00:36:13 tom Exp $
 */

import org.w3c.dom.*;

/**
 * CompressedNode represents a dbXML Compressed Node.  The compression
 * system operates independently of the database and only requires a
 * SymbolTable instance to perform compression/decompression.
 */

public interface CompressedNode extends Node {
   /**
    * isDirty returns whether or not the current Node (or any of its
    * children) have been modified since being loaded.
    *
    * @return Whether or not the Node has been modified
    */
   boolean isDirty();

   /**
    * getDataBytes returns the byte array that defines this Node.  A
    * Node definition is usually a subset of this byte array.
    *
    * @return The Node definition's byte array
    */
   byte[] getDataBytes();

   /**
    * getDataPos returns the offset into the Node definition's byte
    * array that the Node starts at.
    *
    * @return The Node's position
    */
   int getDataPos();

   /**
    * getDataLen returns the length of the Node's definition in the
    * byte array.
    *
    * @return The Node's length
    */
   int getDataLen();

   /**
    * setDataBytes sets the definition and positional information for
    * the Node.
    *
    * @param data The Node definition's byte array
    * @param pos The Node's position
    * @param len The Node's length
    */
   void setDataBytes(byte[] data, int pos, int len);

   /**
    * setDataBytes sets the byte array that defines this Node.
    *
    * @param data The Node definition's byte array
    */
   void setDataBytes(byte[] data);

   /**
    * setDataPos sets the offset into the Node definition's byte array
    * that the Node starts at.
    *
    * @param pos The Node's position
    */
   void setDataPos(int pos);

   /**
    * setDataLen sets the length of the Node's definition in the
    * byte array.
    *
    * @param len The Node's length
    */
   void setDataLen(int len);

   /**
    * load forces the Node to be graphed for this level.  Loading happens
    * automatically whenever a Node's children are accessed, so load
    * should never have to explicitly be called.
    *
    * @throws DOMException if an exception occurs
    */
   void load() throws DOMException;

   /**
    * unload forces the Node to be unloaded for this level.  Whether the
    * node is actually unloaded or not is up to the underlying
    * implementation, and should silently fail if the implementation
    * doesn't support unloading.
    *
    * @throws DOMException if an exception occurs
    */
   void unload() throws DOMException;

   /**
    * isLoaded returns whether or not the Node's definition has been
    * loaded.
    *
    * @return Whether or not the Node is loaded
    */
   boolean isLoaded();

   /**
    * getSymbolID returns the Symbol ID for the current node.  Some nodes
    * will not have a Symbol ID, in which case, a value of -1 is returned.
    *
    * @return The Symbol ID
    */
   short getSymbolID();
}
