package org.dbxml.xml.dom;

/*
 * dbXML License, Version 1.0
 *
 *
 * Copyright (c) 1999-2001 The dbXML Group, L.L.C.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        The dbXML Group (http://www.dbxml.com/)."
 *    Alternately, this acknowledgment may appear in the software
 *    itself, if and wherever such third-party acknowledgments normally
 *    appear.
 *
 * 4. The names "dbXML" and "The dbXML Group" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    info@dbxml.com.
 *
 * 5. Products derived from this software may not be called "dbXML",
 *    nor may "dbXML" appear in their name, without prior written
 *    permission of The dbXML Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE DBXML GROUP OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: ProcessingInstructionImpl.java,v 1.13 2001/08/05 07:23:18 tom Exp $
 */

import org.dbxml.xml.*;
import org.dbxml.server.util.*;

import org.w3c.dom.*;

/**
 * ProcessingInstructionImpl
 */

public final class ProcessingInstructionImpl extends NodeImpl implements ProcessingInstruction {
   public ProcessingInstructionImpl() {
   }
   
   public ProcessingInstructionImpl(NodeImpl parentNode, byte[] data, int pos, int len) {
      super(parentNode, data, pos, len);
   }

   public ProcessingInstructionImpl(NodeImpl parentNode, boolean dirty) {
      super(parentNode, dirty);
   }

   public ProcessingInstructionImpl(NodeImpl parentNode, String nodeName, String nodeValue) {
      super(parentNode, true);
      this.nodeName = nodeName;
      this.nodeValue = nodeValue;
   }

   protected void checkLoaded() {
      if ( loaded )
         return;
      else
         loaded = true;

      try {
         if ( data != null ) {
            DocumentImpl doc = (DocumentImpl)getOwnerDocument();
            SymbolTable st = doc.getSymbols();

            ByteArrayInput bis = new ByteArrayInput(data, pos, len);
            XMLCompressedInput xci = new XMLCompressedInput(bis, st);

            xci.readSignature();      // Skip The Signature
            int size = xci.readInt();

            byte[] buf = new byte[bis.available()];
            xci.read(buf);

            String value = new String(buf);
            int i = value.indexOf(' ');
            nodeName = value.substring(0, i);
            nodeValue = value.substring(i+1);
         }
      }
      catch ( Exception e ) {
         org.dbxml.Debug.printStackTrace(e);
      }
   }

   public short getNodeType() {
      return Node.PROCESSING_INSTRUCTION_NODE;
   }

   public void setNodeValue(String nodeValue) throws DOMException {
      checkLoaded();
      checkReadOnly();
      this.nodeValue = nodeValue;
      setDirty();
   }

   /**
    * The target of this processing instruction. XML defines this as being the
    * first token following the markup that begins the processing instruction.
    */
   public String getTarget() {
      return getNodeName();
   }

   /**
    * The content of this processing instruction. This is from the first non
    * white space character after the target to the character immediately
    * preceding the <code>?&gt;</code>.
    * @exception DOMException
    *   NO_MODIFICATION_ALLOWED_ERR: Raised when the node is readonly.
    */
   public String getData() {
      return getNodeValue();
   }

   public void setData(String data) throws DOMException {
      setNodeValue(data);
   }
}


